//
//
// linear.array.44.c
//
// Charlieplex LED array hello-world
//
// Neil Gershenfeld
// 11/13/10
//
// (c) Massachusetts Institute of Technology 2010
// Permission granted for experimental and personal use;
// license for commercial sale available from MIT.
//

// Modified 4/22/2014 by Scott Zitek

#include <avr/io.h>
#include <util/delay.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define input(directions,pin) (directions &= (~pin)) // set port direction for input 
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set

#define led_delay() _delay_ms(1) // LED delay

#define led_port PORTA
#define led_direction DDRA

#define A (1 << PA1) // row 1
#define B (1 << PA2) // row 2
#define C (1 << PA3) // row 3
#define D (1 << PA4) // row 4
#define E (1 << PA5) // row 5

void flash(uint8_t from, uint8_t to, uint8_t delay) {
   //
   // source from, sink to, flash
   //
   static uint8_t i;
   set(led_port,from);
   clear(led_port,to);
   output(led_direction,from);
   output(led_direction,to);
   for (i = 0; i < delay; ++i)
       led_delay();
   input(led_direction,from);
   input(led_direction,to);
   }

void led_cycle(uint8_t number, uint8_t delay) {
   //
   // cycle through LEDs
   //
   uint8_t i;
   for (i = 0; i < number; ++i) {

	 // LEDs 20 to 1
      flash(D,E,delay);   // LED20
      flash(E,D,delay);   // LED19
      flash(C,D,delay);   // LED18
      flash(D,C,delay);   // LED17
      flash(B,C,delay);   // LED16
      flash(C,B,delay);   // LED15
      flash(A,B,delay);   // LED14
      flash(B,A,delay);   // LED13
      flash(C,E,delay);   // LED12
      flash(E,C,delay);   // LED11
      flash(A,C,delay);   // LED10
      flash(C,A,delay);   // LED9
      flash(B,D,delay);   // LED8
      flash(D,B,delay);   // LED7
      flash(B,E,delay);   // LED6
      flash(E,B,delay);   // LED5
      flash(A,D,delay);   // LED4
      flash(D,A,delay);   // LED3
      flash(A,E,delay);   // LED2
      flash(E,A,delay);   // LED1


      // LEDs 1 to 20
      flash(E,A,delay);   // LED1
      flash(A,E,delay);   // LED2
      flash(D,A,delay);   // LED3
      flash(A,D,delay);   // LED4
      flash(E,B,delay);   // LED5
      flash(B,E,delay);   // LED6
      flash(D,B,delay);   // LED7
      flash(B,D,delay);   // LED8
      flash(C,A,delay);   // LED9
      flash(A,C,delay);   // LED10
      flash(E,C,delay);   // LED11
      flash(C,E,delay);   // LED12
      flash(B,A,delay);   // LED13
      flash(A,B,delay);   // LED14
      flash(C,B,delay);   // LED15
      flash(B,C,delay);   // LED16
      flash(D,C,delay);   // LED17
      flash(C,D,delay);   // LED18
      flash(E,D,delay);   // LED19
      flash(D,E,delay);   // LED20


 	 // LEDs alternate sides towards center
      flash(D,E,delay);   // LED20
      flash(E,A,delay);   // LED1
      flash(E,D,delay);   // LED19
      flash(A,E,delay);   // LED2
      flash(C,D,delay);   // LED18
      flash(D,A,delay);   // LED3
      flash(D,C,delay);   // LED17
      flash(A,D,delay);   // LED4
      flash(B,C,delay);   // LED16
      flash(E,B,delay);   // LED5
      flash(C,B,delay);   // LED15
      flash(B,E,delay);   // LED6
      flash(A,B,delay);   // LED14
      flash(D,B,delay);   // LED7
      flash(B,A,delay);   // LED13
      flash(B,D,delay);   // LED8
      flash(C,E,delay);   // LED12
      flash(C,A,delay);   // LED9
      flash(E,C,delay);   // LED11
      flash(A,C,delay);   // LED10
      

 	 // LEDs alternate sides towards outside
      flash(A,C,delay);   // LED10
      flash(E,C,delay);   // LED11
      flash(C,A,delay);   // LED9
      flash(C,E,delay);   // LED12
      flash(B,D,delay);   // LED8
      flash(B,A,delay);   // LED13
      flash(D,B,delay);   // LED7
      flash(A,B,delay);   // LED14
      flash(B,E,delay);   // LED6
      flash(C,B,delay);   // LED15
      flash(E,B,delay);   // LED5
      flash(B,C,delay);   // LED16
      flash(A,D,delay);   // LED4
      flash(D,C,delay);   // LED17
      flash(D,A,delay);   // LED3
      flash(C,D,delay);   // LED18
      flash(A,E,delay);   // LED2
      flash(E,D,delay);   // LED19
      flash(E,A,delay);   // LED1
      flash(D,E,delay);   // LED20


      }
   }

int main(void) {
   //
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   //
   // main loop
   //
   while (1) {
      led_cycle(1,100);
      led_cycle(3,20);
      led_cycle(5,5);
      led_cycle(10,1);
      }
   }
